# -*- coding: utf-8 -*-
"""FLAction Module."""

from pineboolib.application.metadata import pnaction


class FLAction(pnaction.PNAction):
    """
    FLAction Class.

    This class contains information on actions to open forms.

    It is used to automatically link forms with your script,
    interface and source table.

    @author InfoSiAL S.L.
    """
