# lextab.py. This file automatically created by PLY (version 3.11). Don't edit!
_tabversion   = '3.10'
_lextokens    = set(('AND', 'AT', 'BACKSLASH', 'BREAK', 'CASE', 'CATCH', 'CCONST', 'CLASS', 'COLON', 'COMMA', 'COMMENTCLOSE', 'CONDITIONAL1', 'CONST', 'CONTINUE', 'DEFAULT', 'DELETE', 'DIVEQUAL', 'DIVIDE', 'DO', 'DOCSTRINGOPEN', 'DOLLAR', 'DQOUTE', 'ELSE', 'EQ', 'EQQ', 'EQUALS', 'EXTENDS', 'FCONST', 'FOR', 'FUNCTION', 'GE', 'GT', 'ICONST', 'ID', 'IF', 'IN', 'LAND', 'LBRACE', 'LBRACKET', 'LE', 'LNOT', 'LOR', 'LPAREN', 'LSHIFT', 'LT', 'MINUS', 'MINUSEQUAL', 'MINUSMINUS', 'MOD', 'MODEQUAL', 'NE', 'NEQ', 'NEW', 'OR', 'PERIOD', 'PLUS', 'PLUSEQUAL', 'PLUSPLUS', 'RBRACE', 'RBRACKET', 'RETURN', 'RPAREN', 'RSHIFT', 'SCONST', 'SEMI', 'SQOUTE', 'STATIC', 'SWITCH', 'THROW', 'TIMES', 'TIMESEQUAL', 'TRY', 'TYPEOF', 'VAR', 'WHILE', 'WITH', 'XOR'))
_lexreflags   = 64
_lexliterals  = ''
_lexstateinfo = {'INITIAL': 'inclusive'}
_lexstatere   = {'INITIAL': [('(?P<t_NEWLINE>\\n+)|(?P<t_ID>[A-Za-z_]+[\\w_]*)|(?P<t_comment>(/\\*( |\\*\\*)(.|\\n)*?\\*/)|(//.*))|(?P<t_DOCSTRINGOPEN>/\\*\\*[ ]+)|(?P<t_preprocessor>\\#(.)*?\\n)|(?P<t_FCONST>((\\d+)(\\.\\d+)(e(\\+|-)?(\\d+))? | (\\d+)e(\\+|-)?(\\d+))([lL]|[fF])?)|(?P<t_ICONST>\\d+([uU]|[lL]|[uU][lL]|[lL][uU])?)|(?P<t_CCONST>\\\'([^\\\'\\\\\\n]|(\\\\.)|\\\\\\n)*?\\\')|(?P<t_SCONST>\\"([^\\"\\\\\\n]|(\\\\.)|\\\\\\n)*?\\")|(?P<t_LOR>\\|\\|)|(?P<t_PLUSPLUS>\\+\\+)|(?P<t_COMMENTCLOSE>\\*/)|(?P<t_EQQ>===)|(?P<t_NEQ>!==)|(?P<t_PLUSEQUAL>\\+=)|(?P<t_TIMESEQUAL>\\*=)|(?P<t_BACKSLASH>\\\\)|(?P<t_CONDITIONAL1>\\?)|(?P<t_DIVEQUAL>/=)|(?P<t_DOLLAR>\\$)|(?P<t_EQ>==)|(?P<t_GE>>=)|(?P<t_LAND>&&)|(?P<t_LBRACE>\\{)|(?P<t_LBRACKET>\\[)|(?P<t_LE><=)|(?P<t_LPAREN>\\()|(?P<t_LSHIFT><<)|(?P<t_MINUSEQUAL>-=)|(?P<t_MINUSMINUS>--)|(?P<t_MODEQUAL>%=)|(?P<t_NE>!=)|(?P<t_OR>\\|)|(?P<t_PERIOD>\\.)|(?P<t_PLUS>\\+)|(?P<t_RBRACE>\\})|(?P<t_RBRACKET>\\])|(?P<t_RPAREN>\\))|(?P<t_RSHIFT>>>)|(?P<t_TIMES>\\*)|(?P<t_XOR>\\^)|(?P<t_AND>&)|(?P<t_AT>@)|(?P<t_COLON>:)|(?P<t_COMMA>,)|(?P<t_DIVIDE>/)|(?P<t_DQOUTE>")|(?P<t_EQUALS>=)|(?P<t_GT>>)|(?P<t_LNOT>!)|(?P<t_LT><)|(?P<t_MINUS>-)|(?P<t_MOD>%)|(?P<t_SEMI>;)|(?P<t_SQOUTE>\')', [None, ('t_NEWLINE', 'NEWLINE'), ('t_ID', 'ID'), ('t_comment', 'comment'), None, None, None, None, ('t_DOCSTRINGOPEN', 'DOCSTRINGOPEN'), ('t_preprocessor', 'preprocessor'), None, (None, 'FCONST'), None, None, None, None, None, None, None, None, None, None, (None, 'ICONST'), None, (None, 'CCONST'), None, None, (None, 'SCONST'), None, None, (None, 'LOR'), (None, 'PLUSPLUS'), (None, 'COMMENTCLOSE'), (None, 'EQQ'), (None, 'NEQ'), (None, 'PLUSEQUAL'), (None, 'TIMESEQUAL'), (None, 'BACKSLASH'), (None, 'CONDITIONAL1'), (None, 'DIVEQUAL'), (None, 'DOLLAR'), (None, 'EQ'), (None, 'GE'), (None, 'LAND'), (None, 'LBRACE'), (None, 'LBRACKET'), (None, 'LE'), (None, 'LPAREN'), (None, 'LSHIFT'), (None, 'MINUSEQUAL'), (None, 'MINUSMINUS'), (None, 'MODEQUAL'), (None, 'NE'), (None, 'OR'), (None, 'PERIOD'), (None, 'PLUS'), (None, 'RBRACE'), (None, 'RBRACKET'), (None, 'RPAREN'), (None, 'RSHIFT'), (None, 'TIMES'), (None, 'XOR'), (None, 'AND'), (None, 'AT'), (None, 'COLON'), (None, 'COMMA'), (None, 'DIVIDE'), (None, 'DQOUTE'), (None, 'EQUALS'), (None, 'GT'), (None, 'LNOT'), (None, 'LT'), (None, 'MINUS'), (None, 'MOD'), (None, 'SEMI'), (None, 'SQOUTE')])]}
_lexstateignore = {'INITIAL': ' \r\t\x0c'}
_lexstateerrorf = {'INITIAL': 't_error'}
_lexstateeoff = {}
