# -*- coding: utf-8 -*-
"""
AQSettings Module.

This class provides utilities to access local configuration data.
"""

from pineboolib.fllegacy.flsettings import FLSettings


class AQSettings(FLSettings):
    """AQSettings Class."""

    pass
