# -*- coding: utf-8 -*-
"""
AQBoolFlagState package.

Save cursor states.
"""

from pineboolib.application.acls import pnboolflagstate


class AQBoolFlagState(pnboolflagstate.PNBoolFlagState):
    """AQBollFlagState Class."""

    pass


class AQBoolFlagStateList(pnboolflagstate.PNBoolFlagStateList):
    """AQBoolFlagStateList Class."""

    pass
